package cn.com.taiji.track.es;

import cn.com.taiji.track.config.EsConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.pool2.impl.GenericObjectPool;
import org.elasticsearch.client.RestHighLevelClient;

@Slf4j
public class ElasticSearchPoolUtil {

    private EsConfig esConfig;

    private GenericObjectPool<RestHighLevelClient> genericObjectPool = null;

    public ElasticSearchPoolUtil(EsConfig esConfig){

        this.esConfig = esConfig;

        this.genericObjectPool = EsClientPoolFactory.pool(EsClientPoolFactory.poolFactory(this.esConfig), EsClientPoolFactory.poolConfig(this.esConfig));

    }

    /**
     * 获得对象
     *
     * @return
     * @throws Exception
     */
    public RestHighLevelClient getClient() throws Exception {
        // 从池中取一个对象
        return genericObjectPool.borrowObject();
    }

    /**
     * 归还对象
     *
     * @param client
     */
    public void returnClient(RestHighLevelClient client) throws Exception {
        // 使用完毕之后，归还对象
        genericObjectPool.returnObject(client);
    }

    public void printInfo(){
        log.info("连接池活动数:"+genericObjectPool.getNumActive()+"连接池空闲数:"+genericObjectPool.getNumIdle()+"连接池等待数:"+genericObjectPool.getNumWaiters());
        log.info("连接池获得对象数:"+genericObjectPool.getBorrowedCount()+"连接池创建数:"+genericObjectPool.getCreatedCount()+"连接池连接销毁数:"+genericObjectPool.getDestroyedCount());
    }
}
