package cn.com.taiji.track.es;

import cn.com.taiji.track.config.EsConfig;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.pool2.BasePooledObjectFactory;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.PooledObjectFactory;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.commons.pool2.impl.GenericObjectPool;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;

public class EsClientPoolFactory {

    public static GenericObjectPoolConfig poolConfig(EsConfig esConfig) {

        GenericObjectPoolConfig objectPoolConfig = new GenericObjectPoolConfig();

        objectPoolConfig.setMinIdle(esConfig.getMinIdle());

        objectPoolConfig.setMaxTotal(esConfig.getMaxTotal());

        objectPoolConfig.setMaxIdle(esConfig.getMaxIdle());

        objectPoolConfig.setJmxEnabled(false);

        return objectPoolConfig;
    }

    public static PooledObjectFactory<RestHighLevelClient> poolFactory(EsConfig esConfig) {

        return new BasePooledObjectFactory<RestHighLevelClient>() {

            @Override
            public void destroyObject(PooledObject<RestHighLevelClient> pooledObject) throws Exception {
                RestHighLevelClient highLevelClient = pooledObject.getObject();
                highLevelClient.close();
            }

            @Override
            public RestHighLevelClient create() throws Exception {

                String[] esUris=esConfig.getUris().split(",");

                HttpHost[] httpHosts = new HttpHost[esUris.length];
                //将地址转换为http主机数组，未配置端口则采用默认9200端口，配置了端口则用配置的端口
                for (int i = 0; i < httpHosts.length; i++) {
                    if (!StringUtils.isEmpty(esUris[i])) {
                        if (esUris[i].contains(":")) {
                            String[] uris = esUris[i].split(":");
                            httpHosts[i] = new HttpHost(uris[0], Integer.parseInt(uris[1]), "http");
                        } else {
                            httpHosts[i] = new HttpHost(esUris[i], 9200, "http");
                        }
                    }
                }

                final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
                credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(esConfig.getUsername(), esConfig.getPassword()));

                return new RestHighLevelClient(RestClient.builder(httpHosts)
                        .setRequestConfigCallback(
                                requestConfigBuilder -> {
                                    requestConfigBuilder.setConnectTimeout(esConfig.getConnectionTimeout());
                                    requestConfigBuilder.setSocketTimeout(esConfig.getSocketTimeout());
                                    requestConfigBuilder.setConnectionRequestTimeout(esConfig.getConnectionRequestTimeout());
                                    return requestConfigBuilder;
                                }
                        ).setHttpClientConfigCallback(
                                httpAsyncClientBuilder -> httpAsyncClientBuilder.setDefaultCredentialsProvider(credentialsProvider)
                        )
                );
            }

            @Override
            public PooledObject<RestHighLevelClient> wrap(RestHighLevelClient restHighLevelClient) {
                return new DefaultPooledObject<>(restHighLevelClient);
            }
        };
    }


    public static GenericObjectPool<RestHighLevelClient> pool(
            PooledObjectFactory<RestHighLevelClient> factory,
            GenericObjectPoolConfig config) {
        return new GenericObjectPool<RestHighLevelClient>(factory, config);
    }



}
