package cn.com.taiji.track.es;

import cn.com.taiji.track.entity.ShipTrack;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.geo.GeoPoint;
import org.elasticsearch.index.query.AbstractQueryBuilder;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Slf4j
public class SearchShipTrackUtil {



    public static void searchShipTrack(ElasticSearchPoolUtil esClientPool, String indexInfo, String beginTime, String endTime, int pageNumber, int pageSize,
                                       List<ShipTrack> shipTrackList, Map<String,Long> totalValueMap, String areaType, BigDecimal[] areaLocation) throws Exception{




        RestHighLevelClient restHighLevelClient = null;

        String timeFieldName = "mergeTime";
        String locationFieldName = "location";

        String mergeTargetFieldName = "mergeTarget";

        String[] indexInfos = indexInfo.split("=");

        String indexType = indexInfos[0];

        String indexPrefix = indexInfos[1];

        if("hlx0".equals(indexType) || "index_seat_hlx_ax_original_radar_json_".equals(indexPrefix)){

            indexPrefix = "index_seat_hlx_original_radar_json_";
        }

        String currentIndexName = indexPrefix+beginTime.substring(0,10).trim();

        log.info("索引信息:"+indexInfo+"-----------"+currentIndexName);

        if("fk".equals(indexType)){

            timeFieldName = "mergeTime";

            locationFieldName = "location";

            mergeTargetFieldName = "mergeTarget";

        }else if("beidou".equals(indexType)){

            timeFieldName = "locationTime";
            locationFieldName = "location";
            mergeTargetFieldName = "devideNo";

        }else if("ais".equals(indexType)){

            timeFieldName = "posTimes";
            locationFieldName = "location";

            mergeTargetFieldName = "id";

        }else if("hlx".equals(indexType) || "hlx0".equals(indexType)){

            timeFieldName = "time";
            locationFieldName = "location";

            mergeTargetFieldName = "target_id";

        }else if("tianao".equals(indexType)){

            timeFieldName = "fusionTime";
            locationFieldName = "location";

            mergeTargetFieldName = "fusionBatchNum";

        }

        try {

            restHighLevelClient = esClientPool.getClient();


            SearchRequest searchRequest = new SearchRequest(currentIndexName);

            BoolQueryBuilder boolBuilder = QueryBuilders.boolQuery();
            //初始化查询对象的查询条件
            boolBuilder.filter(QueryBuilders.rangeQuery(timeFieldName).gte(beginTime).lt(endTime));

            AbstractQueryBuilder locationBuilder = null;

            if ("1".equals(areaType)) {
                locationBuilder = QueryBuilders.geoBoundingBoxQuery(locationFieldName)
                        .setCorners(
                                areaLocation[3].doubleValue(),areaLocation[0].doubleValue(),
                                areaLocation[1].doubleValue(), areaLocation[2].doubleValue());
            }else if ("2".equals(areaType)) {

                List<GeoPoint> points = new ArrayList<>();

                for(int i = 0 ;i < areaLocation.length;i=i+2) {

                    points.add(new GeoPoint(areaLocation[i+1].doubleValue(),areaLocation[i].doubleValue()));
                }
                locationBuilder = QueryBuilders.geoPolygonQuery(locationFieldName, points);
            }

            if(locationBuilder !=null){

                boolBuilder.filter(locationBuilder);
            }

            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

            searchSourceBuilder.query(boolBuilder).trackTotalHits(true);

            //设置排序条件
            FieldSortBuilder _mergeTime = SortBuilders.fieldSort(timeFieldName).order(SortOrder.ASC);
            FieldSortBuilder _mergeTarget = SortBuilders.fieldSort(mergeTargetFieldName).order(SortOrder.ASC);

            searchSourceBuilder.sort(_mergeTarget).sort(_mergeTime);

            searchSourceBuilder.from((pageNumber - 1) * Integer.valueOf(pageSize)).size(Integer.valueOf(pageSize));


            searchRequest.source(searchSourceBuilder);

            log.info("根据ID查询es 指定某时间段的历史轨迹预警数据 执行开始................."+searchSourceBuilder.toString());

            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);

            if(searchResponse !=null) {

                SearchHits hits = searchResponse.getHits();

                SearchHit[] searchHits = hits.getHits();


                if(totalValueMap.containsKey("totalValue")){

                    totalValueMap.put("totalValue",hits.getTotalHits().value);

                }

                log.info("分页:"+pageNumber+"--beginTime:"+beginTime+" endTime:"+endTime+"的记录数:"+searchHits.length);

                for (SearchHit hit : searchHits) {

                    ShipTrack st = new ShipTrack();

                    Map<String, Object> sourceAsMap = hit.getSourceAsMap();

                    String _id = hit.getId();

                    if("fk".equals(indexType)){

                        st.setId(sourceAsMap.get("mergeId"));
                        st.setB(sourceAsMap.get("mergeTarget").toString());
                        st.setC(sourceAsMap.get("targetCourse"));
                        st.setS(sourceAsMap.get("targetSpeed"));


                        Object targetNameEn ="";
                        if(sourceAsMap.containsKey("targetNameEn")){

                            targetNameEn = sourceAsMap.get("targetNameEn");

                        }
                        Object targetName = "";
                        if(sourceAsMap.containsKey("targetName")){

                            targetName = sourceAsMap.get("targetName");

                        }

                        if(StringUtils.isNotEmpty(targetName+"")){

                            st.setN(targetName);

                        }else{

                            if(StringUtils.isNotEmpty(targetNameEn+"")){

                                st.setN(targetNameEn);

                            }

                        }

                        st.setLat(sourceAsMap.get("targetLatitude"));
                        st.setLon(sourceAsMap.get("targetLongitude"));

                        String mergeTime = sourceAsMap.get("mergeTime").toString();
                        String mergeTimeData = mergeTime.substring(0,mergeTime.lastIndexOf("."));

                        String formatTime = DateUtil.format(DateUtil.parse(sourceAsMap.get("mergeTime").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN);

//                        log.info(indexType+"-------原始时间:"+mergeTime+"--------截取时间:"+mergeTimeData+"------格式化时间:"+formatTime);

                        //st.setT(DateUtil.format(DateUtil.parse(sourceAsMap.get("mergeTime").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN));

                        st.setT(mergeTimeData);



                        st.setTp(sourceAsMap.get("mergeType"));
                        if(sourceAsMap.containsKey("targetShipType")){

                            st.setStp(sourceAsMap.get("targetShipType"));

                        }

                        st.setL(sourceAsMap.get("targetLength"));

                        if(sourceAsMap.containsKey("targetDraught")){

                            st.setD(sourceAsMap.get("targetDraught"));

                        }


                        if(sourceAsMap.containsKey("targetState")){

                            st.setTs(sourceAsMap.get("targetState"));

                        }

                        st.setTt(sourceAsMap.get("targetType"));
                        st.setTtr(sourceAsMap.get("targetTypeReliability"));
                        st.setGn(sourceAsMap.get("gisNationality"));
                        st.setTno(sourceAsMap.get("trackDeviceNo"));
                        st.setDur(Long.valueOf(sourceAsMap.get("targetTrackTime").toString()));



                    }else if("beidou".equals(indexType)){


                        st.setId(sourceAsMap.get("locationid"));
                        st.setB(sourceAsMap.get("devideNo").toString());
                        st.setC(sourceAsMap.get("course"));
                        st.setS(sourceAsMap.get("speed"));
                        st.setN(sourceAsMap.get("shipName"));
                        st.setLat(sourceAsMap.get("latitude"));
                        st.setLon(sourceAsMap.get("longitude"));
                        st.setT(DateUtil.format(DateUtil.parse(sourceAsMap.get("locationTime").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN));

                        String mergeTime = sourceAsMap.get("locationTime").toString();

                        String mergeTimeData = DateUtil.format(DateUtil.parse(sourceAsMap.get("locationTime").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN);

//                        log.info(indexType+"-------原始时间:"+mergeTime+"--------格式化时间:"+mergeTimeData);

                        st.setTp("BEIDOU");

                        if(sourceAsMap.containsKey("gisShipUse")){

                            st.setStp(sourceAsMap.get("gisShipUse"));

                        }

                        st.setL(sourceAsMap.get("shipLength"));

                        if(sourceAsMap.containsKey("targetDraught")){

                            st.setD(sourceAsMap.get("targetDraught"));

                        }


                        if(sourceAsMap.containsKey("targetState")){

                            st.setTs(sourceAsMap.get("targetState"));

                        }

                        st.setTt("0");
                        st.setTtr("100");
                        st.setGn("412");
                        st.setTno(sourceAsMap.get("devideNo"));


                    }else if("ais".equals(indexType)){

                        st.setId(_id);
                        st.setB(sourceAsMap.get("id").toString());
                        st.setC(sourceAsMap.get("cogs"));
                        st.setLat(sourceAsMap.get("latitude"));
                        st.setLon(sourceAsMap.get("longitude"));
                        st.setT(DateUtil.format(DateUtil.parse(sourceAsMap.get("posTimes").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN));

                        String mergeTime = sourceAsMap.get("posTimes").toString();

                        String mergeTimeData = DateUtil.format(DateUtil.parse(sourceAsMap.get("posTimes").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN);

//                        log.info(indexType+"-------原始时间:"+mergeTime+"--------格式化时间:"+mergeTimeData);

                        st.setTp("ais");
                        if(sourceAsMap.containsKey("dataClass")){

                            st.setStp(sourceAsMap.get("dataClass"));

                        }

                    }else if("hlx".equals(indexType) || "hlx0".equals(indexType)){

                        st.setId(sourceAsMap.get("track_id"));
                        st.setB(sourceAsMap.get("target_id").toString());
                        st.setC(sourceAsMap.get("course"));
                        st.setS(sourceAsMap.get("speed"));
                        st.setLat(sourceAsMap.get("latitude"));
                        st.setLon(sourceAsMap.get("longitude"));
                        st.setT(DateUtil.format(DateUtil.parse(sourceAsMap.get("time").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN));

                        String mergeTime = sourceAsMap.get("time").toString();

                        String mergeTimeData = DateUtil.format(DateUtil.parse(sourceAsMap.get("time").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN);

//                        log.info(indexType+"-------原始时间:"+mergeTime+"--------格式化时间:"+mergeTimeData);

                        st.setTp("RADAR");
                        st.setL(sourceAsMap.get("length"));
                        st.setTt("0");
                        st.setTtr(sourceAsMap.get("reliability"));
                        st.setTno(sourceAsMap.get("radar_id"));


                    }else if("tianao".equals(indexType)){


                        st.setId(sourceAsMap.get("trackId"));
                        st.setB(sourceAsMap.get("fusionBatchNum").toString());
                        st.setC(sourceAsMap.get("course"));
                        st.setS(sourceAsMap.get("speed"));
                        st.setLat(sourceAsMap.get("latitude"));
                        st.setLon(sourceAsMap.get("longitude"));

                        st.setT(DateUtil.format(DateUtil.parse(sourceAsMap.get("fusionTime").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN));

                        String mergeTime = sourceAsMap.get("fusionTime").toString();

                        String mergeTimeData = DateUtil.format(DateUtil.parse(sourceAsMap.get("fusionTime").toString(), DatePattern.NORM_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN);

//                        log.info(indexType+"-------原始时间:"+mergeTime+"--------格式化时间:"+mergeTimeData);

                        st.setTp("RADAR");

                        st.setL(sourceAsMap.get("targetSize"));

                        st.setTt(sourceAsMap.get("targetProper"));
                        st.setTtr(sourceAsMap.get("reliability"));


                    }


                    shipTrackList.add(st);

                }

            }



        }catch (Exception exception){

            exception.printStackTrace();

            log.error("查询es索引:"+currentIndexName+"出现错误！"+exception.getMessage());

        }finally {

            //释放es客户端
            if (restHighLevelClient != null) {

                esClientPool.returnClient(restHighLevelClient);
            }

        }
    }


}
