package cn.com.taiji.track.ws;


import cn.com.taiji.track.config.EsConfig;
import cn.com.taiji.track.es.ElasticSearchPoolUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import io.netty.handler.codec.http.HttpHeaders;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MultiValueMap;
import org.yeauty.annotation.*;
import org.yeauty.pojo.Session;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
public abstract class BaseWsGzip {

    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0;

    //存放websocket session 线程
    public ConcurrentHashMap<String, Thread> concurrentHashMap = new ConcurrentHashMap<String, Thread>();

    //心跳检验
    public volatile boolean isPong;


    @Autowired
    public EsConfig esConfig;

    /**
     * es 客户端线程池
     */
    public ElasticSearchPoolUtil esClientPool = null;


    public BaseWsGzip(){
        this.isPong = true;
    }

    @OnMessage
    public abstract void onMessage(Session session, String message);


    @BeforeHandshake
    public void handshake(Session session, HttpHeaders headers,@RequestParam MultiValueMap reqMap){
        session.setSubprotocols("stomp");
    }

    @OnBinary
    public void onBinary(Session session, byte[] bytes) {
        for (byte b : bytes) {
            System.out.println(b);
        }
        session.sendBinary(bytes);
    }

    @OnEvent
    public void onEvent(Session session, Object evt) {
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent idleStateEvent = (IdleStateEvent) evt;
            switch (idleStateEvent.state()) {
                case READER_IDLE:
                    System.out.println("read idle");
                    break;
                case WRITER_IDLE:
                    System.out.println("write idle");
                    break;
                case ALL_IDLE:
                    System.out.println("all idle");
                    break;
                default:
                    break;
            }
        }
    }

    @OnClose
    public void onClose(Session session){


        try{




            //在线数减1
            if(onlineCount>0) {

                subOnlineCount();

            }

            log.info("gzip-有连接关闭！session id:"+session.channel().id()+",当前在线人数为" + getOnlineCount());


        }catch (Exception exception){

            exception.printStackTrace();

            log.error("pb7z-关闭session id:"+session.channel().id()+"出现异常:"+exception.getMessage());

        }finally {

            session.close();

        }


    }

    @OnError
    public void onError(Session session, Throwable throwable){
        throwable.printStackTrace();
        onClose(session);
    }

    @OnOpen
    public void onOpen(Session session, HttpHeaders headers, @RequestParam MultiValueMap reqMap){

        try{

            if(esClientPool == null ){

                //初始化配置信息
                esClientPool = new ElasticSearchPoolUtil(esConfig);

            }

            //加载轨迹索引文件
//            String filePath = System.getProperty("user.dir")+ File.separator +"index.txt";
//
//            log.info("索引文件路径:"+filePath);
//
//            loadByFilePath(filePath,session);

            //在线数加1
            addOnlineCount();

            log.info("有新连接加入！session id:"+session.channel().id()+",当前在线连接数为" + getOnlineCount());


        }catch (Exception exception){

            exception.printStackTrace();

            log.error("gzip------建立连接onOpen实现异常:"+exception.getMessage());

            session.close();

        }

    }

    private static synchronized int getOnlineCount() {
        return onlineCount;
    }

    private static synchronized void addOnlineCount() {
        BaseWsGzip.onlineCount++;
    }

    private static synchronized void subOnlineCount() {
        BaseWsGzip.onlineCount--;
    }

    private static void loadByFilePath(String filePath,Session session){

        InputStream inputStream = null;

        Assert.notBlank(filePath, "filePath is blank");

        try{

            inputStream = new FileInputStream(filePath);

            if (!ObjectUtil.isNull(inputStream)) {

                Properties properties = new Properties();
                properties.load(inputStream);

                for (String key : properties.stringPropertyNames()) {

                    log.info(key + "=" + properties.getProperty(key));

                    session.setAttribute(key,properties.getProperty(key));
                }

            }else{

                log.error(StrUtil.format("{} is not existed！", filePath));
            }

        }catch (Exception exception){

            exception.printStackTrace();

        }finally {

            if(inputStream !=null){

                try{
                    inputStream.close();

                }catch (Exception e){

                    e.printStackTrace();
                }


            }
        }
    }

}
