package cn.com.taiji.track.ws;


import cn.com.taiji.track.entity.ShipTrack;
import cn.com.taiji.track.es.SearchShipTrackUtil;
import cn.com.taiji.track.page.PageInfo;
import cn.com.taiji.track.util.Gzip;
import cn.hutool.core.date.*;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.yeauty.annotation.OnMessage;
import org.yeauty.annotation.ServerEndpoint;
import org.yeauty.pojo.Session;

import java.math.BigDecimal;
import java.util.*;

@ServerEndpoint(path = "/track-play",host = "${ws.host}",port = "${ws.port}")
@Slf4j
public class TrackPlayWsGzip extends BaseWsGzip {

    @OnMessage
    public void onMessage(Session session, String message) {


        try {

            log.info("gzip----------->客户端发送的信息:" + JSONObject.toJSONString(message));

            if (message != null && !"".equals(message)) {

                JSONObject msgJSONObject = JSONObject.parseObject(message);

                StringBuffer sbStr = new StringBuffer();

                checkNullOrEmpty(msgJSONObject,sbStr);

                if (sbStr.length() ==0) {

                    String[] timeDatas = msgJSONObject.getString("data").split("-");

                    if (timeDatas.length == 2) {


                        String beginTime = DateUtil.format(DateUtil.parse(timeDatas[0], DatePattern.PURE_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN);
                        String endTime = DateUtil.format(DateUtil.parse(timeDatas[1], DatePattern.PURE_DATETIME_PATTERN), DatePattern.NORM_DATETIME_PATTERN);

                        Map<String,Long> totalValueMap = new HashMap<String,Long>();
                        totalValueMap.put("totalValue",0l);

                        int pageNumber =1;
                        int pageSize = 5000;

                        List<ShipTrack> shipTrackList = new ArrayList<ShipTrack>();

                        String indexInfo = msgJSONObject.getString("index");

                        String areaType= msgJSONObject.getString("areaType");

                        JSONArray areaLocationArray= msgJSONObject.getJSONArray("areaLocation");

                        BigDecimal[] areaLocation = areaLocationArray.toArray(new BigDecimal[areaLocationArray.size()]);

                        SearchShipTrackUtil.searchShipTrack(this.esClientPool,indexInfo, beginTime, endTime,pageNumber,pageSize,shipTrackList,totalValueMap,areaType,areaLocation);

                        long totalValue = totalValueMap.get("totalValue");

                        long count = totalValue - shipTrackList.size();

                        if(count >0){

                            long pageCount=0;

                            if (count%pageSize ==0){

                                pageCount = count/pageSize;

                            }else{

                                pageCount = count/pageSize +1;
                            }

                            List<PageInfo> pageInfoList = new ArrayList<PageInfo>();

                            for(int i=0;i<pageCount;i++){

                                PageInfo pageInfo = new PageInfo();
                                pageInfo.setPageNumber(pageNumber+i+1);
                                pageInfo.setPageSize(pageSize);

                                pageInfoList.add(pageInfo);

                            }

                            pageInfoList.parallelStream().forEachOrdered(pageInfo -> {

                                try {

                                    Map<String,Long> totalValueMapTemp = new HashMap<String,Long>();

                                    SearchShipTrackUtil.searchShipTrack(esClientPool, indexInfo, beginTime, endTime, pageInfo.getPageNumber(),pageSize,shipTrackList,totalValueMapTemp,areaType,areaLocation);

                                } catch (Exception e) {

                                    e.printStackTrace();

                                    log.error("并行分页:"+pageInfo.getPageNumber()+"查询es索引前缀:"+indexInfo+"出现错误！"+e.getMessage());

                                }

                            });

                        }



//                        log.info("排序前gzip----------->es 查询数据:"+JSONObject.toJSONString(shipTrackList));

                        log.info(session.channel().id()+"gzip----------->es 查询数据:"+shipTrackList.size());

                        Collections.sort(shipTrackList, new Comparator<ShipTrack>() {
                            @Override
                            public int compare(ShipTrack o1, ShipTrack o2) {

                                int data = o1.getT().compareTo(o2.getT());

                                if(data < 0){

                                    return -1;

                                }else if(data > 0){

                                    return 1;

                                }else{

                                    return 0;
                                }
                            }
                        });

//                        log.info("排序后gzip----------->es 查询数据:"+JSONObject.toJSONString(shipTrackList));


                        long st1 = System.currentTimeMillis();

                        byte[] dataBytes = JSONObject.toJSONBytes(shipTrackList);

                        log.info(session.channel().id()+"转字节数组花费时间:"+(System.currentTimeMillis()-st1));

                        long ct1 = System.currentTimeMillis();

                        byte[] comBytes = Gzip.compress(dataBytes);

                        log.info(session.channel().id()+"压缩字节数组花费时间:"+(System.currentTimeMillis()-ct1)+"大小:"+comBytes.length);

                        long send1 = System.currentTimeMillis();

                        session.sendBinary(comBytes);

                        log.info(session.channel().id()+"ws发送字节数组花费时间:"+(System.currentTimeMillis()-send1));

//                        String data = new String(Gzip.uncompress(Gzip.compress(JSONObject.toJSONBytes(shipTrackList))));
//
//                        log.info("解压:"+data.toString());







                    } else {

                        log.info("gzip----------->客户端发送的信息:" + JSONObject.toJSONString(message) + "时间参数格式不符合要求！");

                        List<ShipTrack> emptyList = new ArrayList<ShipTrack>();

                        session.sendBinary(Gzip.compress(JSONObject.toJSONBytes(emptyList)));
                    }


                } else {

                    log.info("gzip----------->客户端发送的信息:" + JSONObject.toJSONString(message) + "校验信息:"+sbStr.toString());

                    List<ShipTrack> emptyList = new ArrayList<ShipTrack>();

                    session.sendBinary(Gzip.compress(JSONObject.toJSONBytes(emptyList)));

                }

            }

        } catch (Exception exception) {

            List<ShipTrack> emptyList = new ArrayList<ShipTrack>();

            session.sendBinary(Gzip.compress(JSONObject.toJSONBytes(emptyList)));

            exception.printStackTrace();

            log.error("pb7z----session id:" + session.channel().id() + "推送数据出现异常:" + exception.getMessage());


        }
    }


    private void checkNullOrEmpty(JSONObject msgJSONObject,StringBuffer sbStr){

        if(msgJSONObject.containsKey("data")){

           String timeData =  msgJSONObject.getString("data");

            if(timeData ==null || "".equals(timeData)){

                sbStr.append("时间范围参数data为空或者为null");
            }


        }else{

            sbStr.append("没有包含时间范围的参数");
        }

        if(msgJSONObject.containsKey("index")){

            String index =  msgJSONObject.getString("index");

            if(index ==null || "".equals(index)){

                sbStr.append("轨迹索引类型参数index为空或者为null,必须代表fk:俯瞰融合、beidou:北斗、ais:ais、hlx:海兰信、tianao:天奥的一种");
            }


        }else{

            sbStr.append("没有包含轨迹索引类型参数");
        }

        if(msgJSONObject.containsKey("areaType")){

            String areaType =  msgJSONObject.getString("areaType");

            if(areaType ==null || "".equals(areaType)){

                sbStr.append("框选区域类型参数areaType为空或者为null,必须代表框选区域类型 1:矩形 2:自定义多边形的一种");
            }


        }else{

            sbStr.append("没有包含框选区域类型参数");
        }

        if(msgJSONObject.containsKey("areaLocation")){

            JSONArray jSONArray =  msgJSONObject.getJSONArray("areaLocation");

            if(jSONArray ==null || jSONArray.size() ==0){

                sbStr.append("框选区域参数areaLocation为空或者为null");
            }


        }else{

            sbStr.append("没有包含框选区域参数");
        }

    }

}
